const double realmax = 1E300;
const double realmin = 1E-300;
const double PI = 3.14159265358979323846;
#define max(A,B) (A > B ? A : B)
#define min(A,B) (A < B ? A : B)
#define even(x)  ((x & 1) ? 0 : 1)
#define odd(x)  ((x & 1) ? 1 : 0)
#define isint(x) ((x - floor(x)) > 0.0 ? 0 : 1)
#define mat(a, i, j) (*(a + (m*(j)+i)))

double mod(double A, double B){
  double y;
  y = ((double)(A/B - floor(A/B))*B);
  return y;
}

double logLike1D(double *data, double *est, int m, int om, char *noiseType)
{
  double error = 0.0;
  int i;
  if (strcmp(noiseType,"Poisson")==0) {
    for (i = 0; i < m; i++) {
      if ((est[i+om]==0.0) && (data[i+om]==0.0)) {
        error += 0.0;
      }
      else {
        error =  error -est[i+om] +data[i+om]*log(est[i+om]);
      }
    }
  }
  else if (strcmp(noiseType,"Gaussian")==0)
  {
    for (i = 0; i < m; i++) {
      error += (-pow(est[i+om] - data[i+om], 2.0));
    }
  }
  else
    printf("noiseType must be either 'Poisson' or 'Gaussian'.\n");
  return error;
}

double logLike2D(double **data, double **est,int m, int n,int om, int on,char *noiseType)
{
  double error = 0.0;
  int i,j;
  if (strcmp(noiseType,"Poisson")==0) {
    for (i = 0; i < m; i++) {
      for (j = 0; j < n; j++) {
        if ((est[i+om][j+on]==0.0) && (data[i+om][j+on]==0.0)) {
          error += 0.0;
        }
        else {
          error =  error -est[i+om][j+on] +data[i+om][j+on]*log(est[i+om][j+on]);
        }
      }
    }
  }
  else if (strcmp(noiseType,"Gaussian")==0)
  {
    for (i = 0; i < m; i++) {
      for (j = 0; j < n; j++) {
        error += (-pow(est[i+om][j+on] - data[i+om][j+on], 2.0));
      }
    }
  }
  else
    printf("noiseType must be either 'Poisson' or 'Gaussian'.\n");
  return error;
}

double logLike3D(double ***data, double ***est,int m1, int m2, int m3, int o1, int o2, int o3, char *noiseType)
{
  double error = 0.0;
  int i,j,k;
  if (strcmp(noiseType,"Poisson")==0) {
    for (i = 0; i < m1; i++) {
      for (j = 0; j < m2; j++) {
        for (k = 0; k < m3; k++) {
          if ((est[i+o1][j+o2][k+o3]==0.0) && (data[i+o1][j+o2][k+o3]==0.0)) {
            error += 0.0;
          }
          else {
            error =  error -est[i+o1][j+o2][k+o3] +data[i+o1][j+o2][k+o3]*log(est[i+o1][j+o2][k+o3]);
          }
          
        }
      }
    }
  }
  else if (strcmp(noiseType,"Gaussian")==0)
  {
    for (i = 0; i < m1; i++) {
      for (j = 0; j < m2; j++) {
        for (k = 0; k < m3; k++) {
          error += (-pow(est[i+o1][j+o2][k+o3] - data[i+o1][j+o2][k+o3], 2.0));          
        }
      }
    }
  }
  else
    printf("noiseType must be either 'Poisson' or 'Gaussian'.\n");
  return error;
}

